﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text.RegularExpressions;

namespace Dss.Api.Examples.Internal
{
    public static class ExampleListings
    {
        private static readonly Regex DescriptionScrub = new Regex(@"\d+\.\s*");
        private static List<ExampleArea> _exampleAreas;

        public static IEnumerable<ExampleArea> GetExamples()
        {
            if (_exampleAreas == null)
            {
                var types = typeof (ExampleListings).Assembly.GetTypes();
                _exampleAreas = types
                    .Select(t => new {Type = t, Attribute = t.GetCustomAttribute<ExampleAttribute>()})
                    .Where(t => t.Attribute != null)
                    .OrderBy(t => t.Attribute.Description)
                    .Select(t => new ExampleArea(ScrubDescription(t.Attribute.Description), GetClassExamples(t.Type)))
                    .ToList();
            }
            return _exampleAreas;
        }

        private static IEnumerable<Example> GetClassExamples(Type exampleClass)
        {
            var methods = exampleClass.GetMethods();
            return methods
                .Where(m => m.GetCustomAttribute<ExampleAttribute>() != null)
                .Select(m => new Example(m));
        }

        public static string ScrubDescription(string exampleClassDescription)
        {
            if (exampleClassDescription == null)
                return null;
            return DescriptionScrub.Replace(exampleClassDescription, "");
        }
    }
}
